/**
 * Copyright 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 */
package com.sunlabs.archsim.fleet.components.switchFabric;

import com.sunlabs.archsim.base.common.Parameters;
import com.sunlabs.archsim.fleet.common.FleetGlobal;
import com.sunlabs.archsim.fleet.common.Payload;
import com.sunlabs.archsim.gasp.Branch;
import com.sunlabs.archsim.gasp.BranchAction;

/**
 * Branch - two outputs, directs an output to them according to the provided mask.
 * The mask specifies what bit of the address the branch needs to look at to steer the data.
 * @author ib27688
 *
 */
public class SourceMaskBranch extends Branch {
	
	public SourceMaskBranch(String name, Parameters params) {
		super(name, 2, new SourceMaskBranchAction());
		// process the parameters
		if (params != null) {
			for (int i= 0; i<params.size(); i++) {
				String pName= params.getName(i);
				String sValue= params.getValue(i);
				
				if (pName.equals(SwitchFabricDefaults.BRANCH_MASK)) {
					try {
						int mask= Integer.parseInt(sValue);
						((SourceMaskBranchAction)branchAction).setMask(mask);
					}
					catch (Exception e) {
						director.error("Component: " + name + ", id= " + getID() 
								+ ". Bogus value used for " + SwitchFabricDefaults.BRANCH_MASK
								+ ": " + sValue
								+ ". Value is IGNORED");						
					}
				}
				else if (pName.equals(SwitchFabricDefaults.BRANCH_MASK_BIT)
						|| pName.equals(FleetGlobal.PARAMETER) ) {
					try {
						int bit= Integer.parseInt(sValue);
						((SourceMaskBranchAction)branchAction).setMask(1<<bit);
					}
					catch (Exception e) {
						director.error("Component: " + name + ", id= " + getID() 
								+ ". Bogus value used for " + SwitchFabricDefaults.BRANCH_MASK_BIT
								+ ": " + sValue
								+ ". Value is IGNORED");						
					}
				} // else
				else {
					director.error("Component: " + name + ", id= " + getID() 
							+ ": Attribute " + pName + " with value " + sValue
							+ " is ignored");					
				}
				
			}
		} // if params != null
	} // constructor

} // Mask Branch

class SourceMaskBranchAction extends BranchAction {
	
	static final String name= "MaskBranchAction";
	
	protected int mask=1;
	
	public SourceMaskBranchAction() {
		super(name);
	}
	public int setMask(int m) {
		mask= m;
		return mask;
	}
		
	public int selectBranch(Object data) {
		int selection= 0;
		if (data instanceof Payload) {
			Payload d= (Payload)data;
				selection = ((d.source & mask) == 0)? 0 : 1;
		}
		else {
			director.fatalError( name+ " " + getAlias() + ", id= " + getID()
				+ ": Unrecognized object type"
				+ data.getClass().getName() + "instead of Payload");
		}
		return selection;
	}
	
} // MaskBranchAction